/**
 * @file decoder.h
 * @brief Header of the decoding functionality
 * @author Lukas Hostettler
 * @author Roger Fong
 * @version 1.1
 * @date 2015-12-15
 */

#ifndef DECODER_H
#define DECODER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "gridprobability.h"
#include "intpoint.h"

///@brief Resets the rotation decoder to be completely unsure of its state
///@var rot    Pointer to a rotationRecoder structure
void rotationDecoderReset(RotationDecoder* rot);

///@brief Decreases cerainty in its current rotation state
///@var rot     Pointer to a rotationRecoder structure
void rotationDecoderDiminish(RotationDecoder* rot);

/// @brief  Decodes the current points with subgrid accuracy.
/// Finds best area to decode (part of probabilistic treatment)
/// Corrects orientation
/// Decodes
/// Handles subgrid resolution
/// @var    probGrids           Probability grids filled with the dot-offset probabilities
/// @var    pictureCenter       Center of the image(the position to be decoded)
/// @var    rotDec              Current rotationDecoder
/// @var    axes                Grid's main directions in picture coordinates*subdivision
/// @var    dotgridOffsetsInfo  Information about each dot and it's offsets within the refined grid coordinate system
/// @var    divisorForResult    Output will be scaled by this factor
/// @return The position of pictureCenter in the global coordinates of the grid multiplied by divisor for result
IntPoint probGridsDecode(ProbabilityGrids* probGrids, IntPoint pictureCenter, RotationDecoder* rotDec, IntPoint* axes, GridOffsetsInfo* gridOffsetsInfo, IntPoint* sector);

#ifdef __cplusplus
}
#endif

#endif // DECODER_H

