/**
 * @file intlist.h
 * @brief IntList structure header
 * @author Lukas Hostettler
 * @author Roger Fong
 * @version 1.1
 * @date 2015-12-15
 */

#ifndef INTLIST_H
#define INTLIST_H

// We need this for memcpy
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

#pragma GCC push_options
#pragma GCC optimize ("O1")

///@brief   Dynamic list of integers
typedef struct {
    int numElements;//! number of elements in the list
    int * list;     //! pointer to beginning of list
} IntList;

///@brief   Constructor of an IntList
///@return  Empty IntList
static inline IntList intListCreate(int *list){
    IntList newIntList;
    newIntList.list = list;
    newIntList.numElements = 0;
    return newIntList;
}

///@brief   Deep copy of an IntList
///@var     dyn     Pointer to list to be copied
///@return  A deep copy of intlist
static inline IntList intListCopy(IntList* dyn, int *list){
    IntList listCopy = intListCreate(list);
    memcpy(listCopy.list, dyn->list, sizeof(int)*dyn->numElements);
    listCopy.numElements = dyn->numElements;
    return listCopy;
}

///@brief   Add an element at the end of the list
///@var     dyn     Pointer to the IntList to be modified
///@var     Element the integer to be appended to the end of the list
static inline void intListPush(IntList* dyn, int element){
    dyn->list[dyn->numElements++] = element;
}

///@brief Finds the median of a list of integers
///@var list   the integer list in quest
///@return median of list
int intListMedian(IntList list);


#pragma GCC pop_options

#ifdef __cplusplus
}
#endif

#endif // INTLIST_H

